#ifndef BSK_SCENE_ROUTE_H
#define BSK_SCENE_ROUTE_H

#include <basilisk/util/includes.h>
#include <basilisk/scene/virtualScene.h>
#include <basilisk/nodes/virtualNode.h>
#include <basilisk/scene/scene.h>
#include <basilisk/scene/scene2d.h>
#include <basilisk/nodes/node.h>
#include <basilisk/nodes/node2d.h>

#endif