"""
Keyboard key constants
"""
from __future__ import annotations
import typing
__all__: list[str] = ['K_0', 'K_1', 'K_2', 'K_3', 'K_4', 'K_5', 'K_6', 'K_7', 'K_8', 'K_9', 'K_A', 'K_B', 'K_BACKSPACE', 'K_C', 'K_CAPS_LOCK', 'K_D', 'K_E', 'K_ENTER', 'K_ESCAPE', 'K_F', 'K_F1', 'K_F10', 'K_F11', 'K_F12', 'K_F2', 'K_F3', 'K_F4', 'K_F5', 'K_F6', 'K_F7', 'K_F8', 'K_F9', 'K_G', 'K_H', 'K_I', 'K_J', 'K_K', 'K_L', 'K_LEFT_ALT', 'K_LEFT_CONTROL', 'K_LEFT_SHIFT', 'K_M', 'K_N', 'K_O', 'K_P', 'K_Q', 'K_R', 'K_RIGHT_ALT', 'K_RIGHT_CONTROL', 'K_RIGHT_SHIFT', 'K_S', 'K_SPACE', 'K_T', 'K_TAB', 'K_U', 'K_V', 'K_W', 'K_X', 'K_Y', 'K_Z', 'KeyCode']
class KeyCode:
    """
    Members:
    
      K_A
    
      K_B
    
      K_C
    
      K_D
    
      K_E
    
      K_F
    
      K_G
    
      K_H
    
      K_I
    
      K_J
    
      K_K
    
      K_L
    
      K_M
    
      K_N
    
      K_O
    
      K_P
    
      K_Q
    
      K_R
    
      K_S
    
      K_T
    
      K_U
    
      K_V
    
      K_W
    
      K_X
    
      K_Y
    
      K_Z
    
      K_0
    
      K_1
    
      K_2
    
      K_3
    
      K_4
    
      K_5
    
      K_6
    
      K_7
    
      K_8
    
      K_9
    
      K_F1
    
      K_F2
    
      K_F3
    
      K_F4
    
      K_F5
    
      K_F6
    
      K_F7
    
      K_F8
    
      K_F9
    
      K_F10
    
      K_F11
    
      K_F12
    
      K_BACKSPACE
    
      K_TAB
    
      K_ENTER
    
      K_LEFT_SHIFT
    
      K_RIGHT_SHIFT
    
      K_LEFT_CONTROL
    
      K_RIGHT_CONTROL
    
      K_LEFT_ALT
    
      K_RIGHT_ALT
    
      K_CAPS_LOCK
    
      K_ESCAPE
    
      K_SPACE
    """
    K_0: typing.ClassVar[KeyCode]  # value = <KeyCode.K_0: 48>
    K_1: typing.ClassVar[KeyCode]  # value = <KeyCode.K_1: 49>
    K_2: typing.ClassVar[KeyCode]  # value = <KeyCode.K_2: 50>
    K_3: typing.ClassVar[KeyCode]  # value = <KeyCode.K_3: 51>
    K_4: typing.ClassVar[KeyCode]  # value = <KeyCode.K_4: 52>
    K_5: typing.ClassVar[KeyCode]  # value = <KeyCode.K_5: 53>
    K_6: typing.ClassVar[KeyCode]  # value = <KeyCode.K_6: 54>
    K_7: typing.ClassVar[KeyCode]  # value = <KeyCode.K_7: 55>
    K_8: typing.ClassVar[KeyCode]  # value = <KeyCode.K_8: 56>
    K_9: typing.ClassVar[KeyCode]  # value = <KeyCode.K_9: 57>
    K_A: typing.ClassVar[KeyCode]  # value = <KeyCode.K_A: 65>
    K_B: typing.ClassVar[KeyCode]  # value = <KeyCode.K_B: 66>
    K_BACKSPACE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_BACKSPACE: 259>
    K_C: typing.ClassVar[KeyCode]  # value = <KeyCode.K_C: 67>
    K_CAPS_LOCK: typing.ClassVar[KeyCode]  # value = <KeyCode.K_CAPS_LOCK: 280>
    K_D: typing.ClassVar[KeyCode]  # value = <KeyCode.K_D: 68>
    K_E: typing.ClassVar[KeyCode]  # value = <KeyCode.K_E: 69>
    K_ENTER: typing.ClassVar[KeyCode]  # value = <KeyCode.K_ENTER: 257>
    K_ESCAPE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_ESCAPE: 256>
    K_F: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F: 70>
    K_F1: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F1: 290>
    K_F10: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F10: 299>
    K_F11: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F11: 300>
    K_F12: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F12: 301>
    K_F2: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F2: 291>
    K_F3: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F3: 292>
    K_F4: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F4: 293>
    K_F5: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F5: 294>
    K_F6: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F6: 295>
    K_F7: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F7: 296>
    K_F8: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F8: 297>
    K_F9: typing.ClassVar[KeyCode]  # value = <KeyCode.K_F9: 298>
    K_G: typing.ClassVar[KeyCode]  # value = <KeyCode.K_G: 71>
    K_H: typing.ClassVar[KeyCode]  # value = <KeyCode.K_H: 72>
    K_I: typing.ClassVar[KeyCode]  # value = <KeyCode.K_I: 73>
    K_J: typing.ClassVar[KeyCode]  # value = <KeyCode.K_J: 74>
    K_K: typing.ClassVar[KeyCode]  # value = <KeyCode.K_K: 75>
    K_L: typing.ClassVar[KeyCode]  # value = <KeyCode.K_L: 76>
    K_LEFT_ALT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_ALT: 342>
    K_LEFT_CONTROL: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_CONTROL: 341>
    K_LEFT_SHIFT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_LEFT_SHIFT: 340>
    K_M: typing.ClassVar[KeyCode]  # value = <KeyCode.K_M: 77>
    K_N: typing.ClassVar[KeyCode]  # value = <KeyCode.K_N: 78>
    K_O: typing.ClassVar[KeyCode]  # value = <KeyCode.K_O: 79>
    K_P: typing.ClassVar[KeyCode]  # value = <KeyCode.K_P: 80>
    K_Q: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Q: 81>
    K_R: typing.ClassVar[KeyCode]  # value = <KeyCode.K_R: 82>
    K_RIGHT_ALT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_ALT: 346>
    K_RIGHT_CONTROL: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_CONTROL: 345>
    K_RIGHT_SHIFT: typing.ClassVar[KeyCode]  # value = <KeyCode.K_RIGHT_SHIFT: 344>
    K_S: typing.ClassVar[KeyCode]  # value = <KeyCode.K_S: 83>
    K_SPACE: typing.ClassVar[KeyCode]  # value = <KeyCode.K_SPACE: 32>
    K_T: typing.ClassVar[KeyCode]  # value = <KeyCode.K_T: 84>
    K_TAB: typing.ClassVar[KeyCode]  # value = <KeyCode.K_TAB: 258>
    K_U: typing.ClassVar[KeyCode]  # value = <KeyCode.K_U: 85>
    K_V: typing.ClassVar[KeyCode]  # value = <KeyCode.K_V: 86>
    K_W: typing.ClassVar[KeyCode]  # value = <KeyCode.K_W: 87>
    K_X: typing.ClassVar[KeyCode]  # value = <KeyCode.K_X: 88>
    K_Y: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Y: 89>
    K_Z: typing.ClassVar[KeyCode]  # value = <KeyCode.K_Z: 90>
    __members__: typing.ClassVar[dict[str, KeyCode]]  # value = {'K_A': <KeyCode.K_A: 65>, 'K_B': <KeyCode.K_B: 66>, 'K_C': <KeyCode.K_C: 67>, 'K_D': <KeyCode.K_D: 68>, 'K_E': <KeyCode.K_E: 69>, 'K_F': <KeyCode.K_F: 70>, 'K_G': <KeyCode.K_G: 71>, 'K_H': <KeyCode.K_H: 72>, 'K_I': <KeyCode.K_I: 73>, 'K_J': <KeyCode.K_J: 74>, 'K_K': <KeyCode.K_K: 75>, 'K_L': <KeyCode.K_L: 76>, 'K_M': <KeyCode.K_M: 77>, 'K_N': <KeyCode.K_N: 78>, 'K_O': <KeyCode.K_O: 79>, 'K_P': <KeyCode.K_P: 80>, 'K_Q': <KeyCode.K_Q: 81>, 'K_R': <KeyCode.K_R: 82>, 'K_S': <KeyCode.K_S: 83>, 'K_T': <KeyCode.K_T: 84>, 'K_U': <KeyCode.K_U: 85>, 'K_V': <KeyCode.K_V: 86>, 'K_W': <KeyCode.K_W: 87>, 'K_X': <KeyCode.K_X: 88>, 'K_Y': <KeyCode.K_Y: 89>, 'K_Z': <KeyCode.K_Z: 90>, 'K_0': <KeyCode.K_0: 48>, 'K_1': <KeyCode.K_1: 49>, 'K_2': <KeyCode.K_2: 50>, 'K_3': <KeyCode.K_3: 51>, 'K_4': <KeyCode.K_4: 52>, 'K_5': <KeyCode.K_5: 53>, 'K_6': <KeyCode.K_6: 54>, 'K_7': <KeyCode.K_7: 55>, 'K_8': <KeyCode.K_8: 56>, 'K_9': <KeyCode.K_9: 57>, 'K_F1': <KeyCode.K_F1: 290>, 'K_F2': <KeyCode.K_F2: 291>, 'K_F3': <KeyCode.K_F3: 292>, 'K_F4': <KeyCode.K_F4: 293>, 'K_F5': <KeyCode.K_F5: 294>, 'K_F6': <KeyCode.K_F6: 295>, 'K_F7': <KeyCode.K_F7: 296>, 'K_F8': <KeyCode.K_F8: 297>, 'K_F9': <KeyCode.K_F9: 298>, 'K_F10': <KeyCode.K_F10: 299>, 'K_F11': <KeyCode.K_F11: 300>, 'K_F12': <KeyCode.K_F12: 301>, 'K_BACKSPACE': <KeyCode.K_BACKSPACE: 259>, 'K_TAB': <KeyCode.K_TAB: 258>, 'K_ENTER': <KeyCode.K_ENTER: 257>, 'K_LEFT_SHIFT': <KeyCode.K_LEFT_SHIFT: 340>, 'K_RIGHT_SHIFT': <KeyCode.K_RIGHT_SHIFT: 344>, 'K_LEFT_CONTROL': <KeyCode.K_LEFT_CONTROL: 341>, 'K_RIGHT_CONTROL': <KeyCode.K_RIGHT_CONTROL: 345>, 'K_LEFT_ALT': <KeyCode.K_LEFT_ALT: 342>, 'K_RIGHT_ALT': <KeyCode.K_RIGHT_ALT: 346>, 'K_CAPS_LOCK': <KeyCode.K_CAPS_LOCK: 280>, 'K_ESCAPE': <KeyCode.K_ESCAPE: 256>, 'K_SPACE': <KeyCode.K_SPACE: 32>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: typing.SupportsInt) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: typing.SupportsInt) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
K_0: KeyCode  # value = <KeyCode.K_0: 48>
K_1: KeyCode  # value = <KeyCode.K_1: 49>
K_2: KeyCode  # value = <KeyCode.K_2: 50>
K_3: KeyCode  # value = <KeyCode.K_3: 51>
K_4: KeyCode  # value = <KeyCode.K_4: 52>
K_5: KeyCode  # value = <KeyCode.K_5: 53>
K_6: KeyCode  # value = <KeyCode.K_6: 54>
K_7: KeyCode  # value = <KeyCode.K_7: 55>
K_8: KeyCode  # value = <KeyCode.K_8: 56>
K_9: KeyCode  # value = <KeyCode.K_9: 57>
K_A: KeyCode  # value = <KeyCode.K_A: 65>
K_B: KeyCode  # value = <KeyCode.K_B: 66>
K_BACKSPACE: KeyCode  # value = <KeyCode.K_BACKSPACE: 259>
K_C: KeyCode  # value = <KeyCode.K_C: 67>
K_CAPS_LOCK: KeyCode  # value = <KeyCode.K_CAPS_LOCK: 280>
K_D: KeyCode  # value = <KeyCode.K_D: 68>
K_E: KeyCode  # value = <KeyCode.K_E: 69>
K_ENTER: KeyCode  # value = <KeyCode.K_ENTER: 257>
K_ESCAPE: KeyCode  # value = <KeyCode.K_ESCAPE: 256>
K_F: KeyCode  # value = <KeyCode.K_F: 70>
K_F1: KeyCode  # value = <KeyCode.K_F1: 290>
K_F10: KeyCode  # value = <KeyCode.K_F10: 299>
K_F11: KeyCode  # value = <KeyCode.K_F11: 300>
K_F12: KeyCode  # value = <KeyCode.K_F12: 301>
K_F2: KeyCode  # value = <KeyCode.K_F2: 291>
K_F3: KeyCode  # value = <KeyCode.K_F3: 292>
K_F4: KeyCode  # value = <KeyCode.K_F4: 293>
K_F5: KeyCode  # value = <KeyCode.K_F5: 294>
K_F6: KeyCode  # value = <KeyCode.K_F6: 295>
K_F7: KeyCode  # value = <KeyCode.K_F7: 296>
K_F8: KeyCode  # value = <KeyCode.K_F8: 297>
K_F9: KeyCode  # value = <KeyCode.K_F9: 298>
K_G: KeyCode  # value = <KeyCode.K_G: 71>
K_H: KeyCode  # value = <KeyCode.K_H: 72>
K_I: KeyCode  # value = <KeyCode.K_I: 73>
K_J: KeyCode  # value = <KeyCode.K_J: 74>
K_K: KeyCode  # value = <KeyCode.K_K: 75>
K_L: KeyCode  # value = <KeyCode.K_L: 76>
K_LEFT_ALT: KeyCode  # value = <KeyCode.K_LEFT_ALT: 342>
K_LEFT_CONTROL: KeyCode  # value = <KeyCode.K_LEFT_CONTROL: 341>
K_LEFT_SHIFT: KeyCode  # value = <KeyCode.K_LEFT_SHIFT: 340>
K_M: KeyCode  # value = <KeyCode.K_M: 77>
K_N: KeyCode  # value = <KeyCode.K_N: 78>
K_O: KeyCode  # value = <KeyCode.K_O: 79>
K_P: KeyCode  # value = <KeyCode.K_P: 80>
K_Q: KeyCode  # value = <KeyCode.K_Q: 81>
K_R: KeyCode  # value = <KeyCode.K_R: 82>
K_RIGHT_ALT: KeyCode  # value = <KeyCode.K_RIGHT_ALT: 346>
K_RIGHT_CONTROL: KeyCode  # value = <KeyCode.K_RIGHT_CONTROL: 345>
K_RIGHT_SHIFT: KeyCode  # value = <KeyCode.K_RIGHT_SHIFT: 344>
K_S: KeyCode  # value = <KeyCode.K_S: 83>
K_SPACE: KeyCode  # value = <KeyCode.K_SPACE: 32>
K_T: KeyCode  # value = <KeyCode.K_T: 84>
K_TAB: KeyCode  # value = <KeyCode.K_TAB: 258>
K_U: KeyCode  # value = <KeyCode.K_U: 85>
K_V: KeyCode  # value = <KeyCode.K_V: 86>
K_W: KeyCode  # value = <KeyCode.K_W: 87>
K_X: KeyCode  # value = <KeyCode.K_X: 88>
K_Y: KeyCode  # value = <KeyCode.K_Y: 89>
K_Z: KeyCode  # value = <KeyCode.K_Z: 90>
